@@ -14,20 +14,21 @@
 
 
 """
-qermit provides tools for running and composing error-mitigation methods. Error 
-mitigation methods are split into two types, those which in some manner modify the set 
+qermit provides tools for running and composing error-mitigation methods. Error
+mitigation methods are split into two types, those which in some manner modify the set
 of shots returned when running quantum circuits on quantum devices (MitRes), and those which
 modify the expectation value of some observable (MitEx).
 """
-from qermit.taskgraph.task_graph import TaskGraph
+
+from qermit.taskgraph.mitex import MitEx
+from qermit.taskgraph.mitres import MitRes
 from qermit.taskgraph.mittask import (
-    MitTask,
     AnsatzCircuit,
     CircuitShots,
+    MitTask,
     ObservableExperiment,
 )
-from qermit.taskgraph.mitres import MitRes
-from qermit.taskgraph.mitex import MitEx
-from qermit.taskgraph.utils import SymbolsDict, MeasurementCircuit, ObservableTracker
+from qermit.taskgraph.task_graph import TaskGraph
+from qermit.taskgraph.utils import MeasurementCircuit, ObservableTracker, SymbolsDict
 
-__path__ = __import__("pkgutil").extend_path(__path__, __name__)  # type: ignore
+__path__ = __import__("pkgutil").extend_path(__path__, __name__)
