@@ -13,29 +13,29 @@
 # limitations under the License.
 
 
+from copy import copy
+from typing import Dict, List, Tuple, Union, cast
+
+from numpy import mean
 from pytket import Circuit
 from pytket.backends import Backend
-from pytket.transform import Transform  # type: ignore
+from pytket.pauli import Pauli, QubitPauliString
+from pytket.tailoring import apply_clifford_basis_change
+from pytket.transform import Transform
 from pytket.utils import QubitPauliOperator
-from pytket.pauli import Pauli, QubitPauliString  # type: ignore
-from pytket.tailoring import apply_clifford_basis_change  # type: ignore
-from numpy import mean  # type: ignore
-
-from typing import List, Tuple, cast, Union, Dict
-from copy import copy
-from sympy.core.expr import Expr
+from sympy.core.expr import Expr  # type: ignore
 
 from qermit import (
-    MitEx,
-    SymbolsDict,
-    MeasurementCircuit,
-    ObservableTracker,
-    MitTask,
     AnsatzCircuit,
+    MeasurementCircuit,
+    MitEx,
+    MitTask,
     ObservableExperiment,
+    ObservableTracker,
+    SymbolsDict,
     TaskGraph,
 )
-from qermit.taskgraph.mitex import get_basic_measurement_circuit, gen_compiled_MitRes
+from qermit.taskgraph.mitex import gen_compiled_MitRes, get_basic_measurement_circuit
 
 
 def get_clifford_mcs(input_circuit: Circuit) -> List[MeasurementCircuit]:
@@ -45,10 +45,8 @@ def get_clifford_mcs(input_circuit: Circuit) -> List[MeasurementCircuit]:
     object varies with some different set of Clifford angles.
 
     :param input_circuit: Circuit to make all-non Clifford gates paramterised.
-    :type input_circuit: Circuit
 
     :return: New MeasurementCircuits with Clifford parameters
-    :rtype: List[MeasurementCircuit]
     """
     copy_circ = input_circuit.copy()
     symbols = copy_circ.free_symbols()
@@ -96,20 +94,20 @@ def DFSC_circuit_task_gen() -> MitTask:
     added for each Qubit Pauli String in the operator.
 
     :return: MitTask object that produces characterisation circuits for DFSC on a new wire as new experiments
-    :rtype: MitTask
     """
 
     def task(
         obj,
         measurement_wires: List[ObservableExperiment],
-    ) -> Tuple[List[ObservableExperiment], List[List[List[ObservableExperiment]]],]:
+    ) -> Tuple[
+        List[ObservableExperiment],
+        List[List[List[ObservableExperiment]]],
+    ]:
         """
         :param measurement_wires: A list of tuples, each tuple representing a different experiment
-        :type measurement_wires: List[ObservableExperiment]
 
         :return: Original experiment wires and another list of characterisation experiments for each original experiment.
         These are organised in later task.
-        :rtype: Tuple[List[ObservableExperiment], List[List[List[ObservableExperiment]]]]
         """
         characterisation_wires = []
         for measurement_wire in measurement_wires:
@@ -178,7 +176,6 @@ def DFSC_collater_task_gen() -> MitTask:
     :return: MitTask object that collates many BackendResult objects for a single
         frame randomisation instance and converts them into a single
         BackendResult object.
-    :rtype: MitTask
     """
 
     def task(
@@ -188,11 +185,9 @@ def DFSC_collater_task_gen() -> MitTask:
         """
         :param all_characterisation_trackers: Experiment wires; outer list is experiments, second outer list
         is Cliffords, inner list is qubit pauli strings.
-        :type all_characterisation_trackers: List[List[List[ObservableExperiment]]]
 
         :return: Wire 1; All individual experiments collated into a single wire.
         Wire 2; Indexing to produce characterisation later.
-        :rtype: Tuple[List[ObservableExperiment], List[Tuple[int, List[int]]]]
         """
         organisation_indices = []
         collated_experiments = []
@@ -216,7 +211,6 @@ def DFSC_characterisation_task_gen() -> MitTask:
     a characterisation result for each Experiment.
 
     :return: MitTask object for organising and calculating characterisation.
-    :rtype: MitTask
     """
 
     def task(
@@ -226,12 +220,9 @@ def DFSC_characterisation_task_gen() -> MitTask:
     ) -> Tuple[List[QubitPauliOperator]]:
         """
         :param characterisation_results: All QubitPauliOperators returned from running experiment through some MitEx object
-        :type characteriastion_results: List[QubitPauliOperator]
         :param experiment_indexing: Number of characteriastion results for each experiment, used to split results up.
-        :type experiment_indexing: List[int]
 
         :return: Collated characterisation results, one QubitPauliOperator characterisation for each experiment
-        :rtype: Tuple[List[QubitPauliOperator]]
         """
         split_results = []
         lower_bound = 0
@@ -248,7 +239,7 @@ def DFSC_characterisation_task_gen() -> MitTask:
                 for string in qpo._dict:
                     if string not in characterisation_dict:
                         characterisation_dict[string] = list()
-                    characterisation_dict[string].append(qpo._dict[string])
+                    characterisation_dict[string].append(float(qpo._dict[string]))
             # set entry to average of values in list
             # add characterisation for DFSC to output list
             characterisation_qpos.append(
@@ -270,13 +261,11 @@ def DFSC_characterisation_task_gen() -> MitTask:
 def DFSC_correction_task_gen(zero_threshold: float) -> MitTask:
     """
     For each experiment expectation, if characterisation value greater than threshold, divide experiment expectation
-    by characteriastion value to correct for depolaring noise.
+    by characterisation value to correct for depolarising noise.
 
-    :param zero_threshold: Method does not correct for zero characteriastion expectation values, threshold for this zero limit.
-    :type zero_threshold: float
+    :param zero_threshold: Method does not correct for zero characterisation expectation values, threshold for this zero limit.
 
-    :return: Function for DFSC correctoin.
-    :rtype: MitTask
+    :return: Function for DFSC correction.
     """
 
     def task(
@@ -286,12 +275,9 @@ def DFSC_correction_task_gen(zero_threshold: float) -> MitTask:
     ) -> Tuple[List[QubitPauliOperator]]:
         """
         :param experiment_results: QubitPauliOperators corresponding to expectations for all observable experiments.
-        :type experiment_results: List[QubitPauliOperator]
         :param characteriastion_results: QubitPauliOperators corresponding to expectations for all characterisation experiments.
-        :type characterisation_results: List[QubitPauliOperator]
 
         :return: Corrected expectations as QubitPauliOperator objects.
-        :rtype: Tuple[List[QubitPauliOperator]]
         """
         if len(experiment_results) != len(characterisation_results):
             raise ValueError(
@@ -321,13 +307,10 @@ def gen_DFSC_MitEx(backend: Backend, **kwargs) -> MitEx:
     Produces a MitEx object that applies DFSC characterisation to all experiment results.
 
     :param backend: Backend experiments are run through.
-    :type backend: Backend
     :key experiment_mitex: MitEx object observable experiments are run through
     :key characterisation_mitex: MitEX object characteriastion experiments are run through.
 
     :return: MitEx object for automatic DFSC correction of circuits.
-    :rtype: MitEx
-
     """
 
     _experiment_mitex = copy(
