@@ -13,29 +13,30 @@
 # limitations under the License.
 
 
-from typing import List, Tuple, Counter, cast, Sequence
-from functools import reduce
 import operator
-
-from qermit import (
-    MitRes,
-    MitTask,
-    CircuitShots,
-)
-from qermit.taskgraph.mitex import backend_compile_circuit_shots_task_gen
-from pytket import Circuit, Bit
 from copy import copy
+from enum import Enum
+from functools import reduce
 from math import ceil
+from typing import Counter, List, Sequence, Tuple, cast
+
+from pytket import Bit, Circuit, OpType
 from pytket.backends import Backend
 from pytket.backends.backendresult import BackendResult
+from pytket.passes import AutoRebase
+from pytket.tailoring import PauliFrameRandomisation, UniversalFrameRandomisation
 from pytket.utils.outcomearray import OutcomeArray
-from pytket.tailoring import UniversalFrameRandomisation, PauliFrameRandomisation  # type: ignore
-from enum import Enum
-from pytket import OpType
-from pytket.passes import auto_rebase_pass
+
+from qermit import (
+    CircuitShots,
+    MitRes,
+    MitTask,
+)
+from qermit.taskgraph.mitex import backend_compile_circuit_shots_task_gen
 
 
 class FrameRandomisation(Enum):
+    @staticmethod
     def PauliFrameRandomisation(
         circuit: Circuit, shots: int, samples: int
     ) -> List[CircuitShots]:
@@ -44,18 +45,16 @@ class FrameRandomisation(Enum):
         circuits corresponding to instances of frame randomisation for the input circuit.
 
         :param circuit: Circuit to have frame randomisation applied.
-        :type circuit: Circuit
         :param shots: Number of shots of each frame randomisation circuit to take.
-        :type shots: int
         :param samples: Number of frame randomisation instances to sample.
-        :type samples: int
         """
         pfr = PauliFrameRandomisation()
         pfr_shots = ceil(shots / samples)
-        auto_rebase_pass({OpType.CX, OpType.Rz, OpType.H, OpType.S}).apply(circuit)
+        AutoRebase({OpType.CX, OpType.Rz, OpType.H, OpType.S}).apply(circuit)
         pfr_circuits = pfr.sample_circuits(circuit, samples)
         return [CircuitShots(Circuit=c, Shots=pfr_shots) for c in pfr_circuits]
 
+    @staticmethod
     def UniversalFrameRandomisation(
         circuit: Circuit, shots: int, samples: int
     ) -> List[CircuitShots]:
@@ -68,16 +67,13 @@ class FrameRandomisation(Enum):
         that Rz(-x) and Rz(x) incur similar noise for any angle x.
 
         :param circuit: Circuit to have frame randomisation applied.
-        :type circuit: Circuit
         :param shots: Number of shots of each frame randomisation circuit to take.
-        :type shots: int
         :param samples: Number of frame randomisation instances to sample.
-        :type samples: int
         """
         ufr = UniversalFrameRandomisation()
         ufr_shots = ceil(shots / samples)
 
-        auto_rebase_pass({OpType.CX, OpType.Rz, OpType.H}).apply(circuit)
+        AutoRebase({OpType.CX, OpType.Rz, OpType.H}).apply(circuit)
         ufr_circuits = ufr.sample_circuits(circuit, samples)
         return [CircuitShots(Circuit=c, Shots=ufr_shots) for c in ufr_circuits]
 
@@ -89,17 +85,14 @@ def frame_randomisation_circuits_task_gen(
     Returns a MitTask object that produces Frame Randomisation circuits for some wire of experiment circuits.
 
     :param samples: Number of samples of frame randomisation circuits to take for each circuit in wire.
-    :type samples: int
     """
 
     def task(obj, circs_shots: List[CircuitShots]) -> Tuple[List[CircuitShots]]:
         """
         :param circ_shots: A list of tuple of circuits and shots. Each circuit has frame randomisation applied.
         The number of shots of each frame randomisation circuit is ceil(shots/samples)
-        :type circ_shots: List[CircuiShots]
 
         :return: Frame Randomisation circutis
-        :rtype: Tuple[List[CircuitShots]]
         """
 
         all_fr_circs_shots = []
@@ -125,7 +118,6 @@ def frame_randomisation_result_task_gen(samples: int) -> MitTask:
 
     :param samples: Number of frame randomisation instances created in the first place, used to suitably
         collate results.
-    :type samples: int
     """
 
     def task(
@@ -134,10 +126,8 @@ def frame_randomisation_result_task_gen(samples: int) -> MitTask:
     ) -> Tuple[List[BackendResult]]:
         """
         :param all_fr_results: A list of BackendResult objects for all frame randomisations for all experiment circuits.
-        :type: all_fr_results: List[BackendResult]
 
         :return: Collated BackendResult objects, reflecting frame randomisation procedure
-        :rtype: Tuple[List[BackendResult]]
         """
         chunked_results = [
             all_fr_results[i : i + samples]
@@ -172,9 +162,7 @@ def gen_Frame_Randomisation_MitRes(backend: Backend, samples: int, **kwargs) ->
     Produces a MitRes object that applies FrameRandomisation techniques to experiment circuits.
 
     :param backend: Backend which experiments are default run through.
-    :type backend: Backend
     :param samples: Number of Frame Randomisation instances sampled in Frame Randomisation.
-    :type samples: int
 
     :key mitres: MitRes object FrameRandomisation MitRes built around if given.
     :key frame_randomisation: FrameRandomisation Enum passed to specify method used.
