@@ -14,18 +14,19 @@
 
 
 import itertools
-from functools import lru_cache
-
-from pytket.circuit import Circuit, Qubit, Bit, Node, CircBox  # type: ignore
-import numpy as np  # type: ignore
 from collections import OrderedDict, namedtuple
-from typing import Dict, Iterable, List, Tuple, Counter, cast, Optional
-from math import ceil, log2
-from pytket.backends import Backend
-from pytket.passes import DecomposeBoxes, FlattenRegisters  # type: ignore
-from pytket.backends.backendresult import BackendResult
-from pytket.utils.outcomearray import OutcomeArray
 from enum import Enum
+from functools import lru_cache
+from math import ceil, log2
+from typing import Counter, Dict, Iterable, List, Optional, Tuple, cast
+
+import numpy as np
+from pytket.backends import Backend
+from pytket.backends.backendresult import BackendResult
+from pytket.circuit import Bit, CircBox, Circuit, Node, Qubit
+from pytket.passes import DecomposeBoxes, FlattenRegisters
+from pytket.unit_id import UnitID
+from pytket.utils.outcomearray import OutcomeArray
 
 FullCorrelatedNoiseCharacterisation = namedtuple(
     "FullCorrelatedNoiseCharacterisation",
@@ -34,6 +35,7 @@ FullCorrelatedNoiseCharacterisation = namedtuple(
 
 StateInfo = namedtuple("StateInfo", ["PreparedStates", "QubitBitMaps"])
 
+
 # Helper methods for holding basis states
 @lru_cache(maxsize=128)
 def binary_to_int(bintuple: Tuple[int]) -> int:
@@ -41,9 +43,7 @@ def binary_to_int(bintuple: Tuple[int]) -> int:
     first element of tuple.
 
     :param bintuple: Binary tuple
-    :type bintuple: Tuple[int]
     :return: Integer
-    :rtype: int
     """
     integer = 0
     for index, bitset in enumerate(reversed(bintuple)):
@@ -58,11 +58,8 @@ def int_to_binary(val: int, dim: int) -> Tuple[int, ...]:
     the first element of tuple.
 
     :param val: input integer
-    :type val: int
     :param dim: Bit width
-    :type dim: int
     :return: Binary tuple of width dim
-    :rtype: Tuple[int, ...]
     """
     return tuple(map(int, format(val, "0{}b".format(dim))))
 
@@ -74,24 +71,23 @@ def get_full_transition_tomography_circuits(
 
     :param circuit: Circuit surmising correlated noise process being characterised
     :param backend: Backend on which the experiments are run.
-    :type backend: Backend
     :param correlations: A list of lists of correlated Nodes of a `Device`.
         Qubits within the same list are assumed to only have errors correlated
         with each other. Thus to allow errors between all qubits you should
         provide a single list.  The qubits in `correlations` must be nodes in the
         backend's associated `Device`.
-    :type correlations: List[List[Node]]
     :return: A list of calibration circuits to be run on the machine. The circuits
         should be processed without compilation.
-    :rtype: List[Circuit]
     """
 
+    def to_tuple(correlation_list: List[Node]) -> Tuple[Node, ...]:
+        return tuple(correlation_list)
+
     subsets_matrix_map = OrderedDict.fromkeys(
-        sorted(map(tuple, correlations), key=len, reverse=True)
+        sorted(map(to_tuple, correlations), key=len, reverse=True)
     )
     # ordered from largest to smallest via OrderedDict & sorted
     subset_dimensions = [len(subset) for subset in subsets_matrix_map]
-
     major_state_dimensions = subset_dimensions[0]
     n_circuits = 1 << major_state_dimensions
     all_qubits = [qb for subset in correlations for qb in subset]
@@ -109,16 +105,25 @@ def get_full_transition_tomography_circuits(
 
     # set up CircBox of X gate for preparing basis states
     xcirc = Circuit(1).X(0)
-    xcirc = backend.get_compiled_circuit(xcirc)
+    xcirc = backend.get_compiled_circuit(xcirc, optimisation_level=0)
     FlattenRegisters().apply(xcirc)
     xbox = CircBox(xcirc)
-
     # need to be default register to add as box suitably
-    process_circuit = backend.get_compiled_circuit(process_circuit)
+
+    n_qubits_pre_compile = process_circuit.n_qubits
+    # This needs to be optimisation level 0 to avoid using simplify initial
+    process_circuit = backend.get_compiled_circuit(
+        process_circuit, optimisation_level=0
+    )
+
+    while process_circuit.n_qubits < n_qubits_pre_compile:
+        process_circuit.add_qubit(Qubit("temp_q", process_circuit.n_qubits))
+
     rename_map_pc = {}
     for index, qb in enumerate(process_circuit.qubits):
         rename_map_pc[qb] = Qubit(index)
-    process_circuit.rename_units(rename_map_pc)
+    process_circuit.rename_units(cast(Dict[UnitID, UnitID], rename_map_pc))
+
     pbox = CircBox(process_circuit)
 
     # set up base circuit for appending xbox to
@@ -144,13 +149,14 @@ def get_full_transition_tomography_circuits(
             new_state_dicts[qubits] = major_state[:dim]
             # find only qubits that are expected to be in 1 state, add xbox to given qubits
             for flipped_qb in itertools.compress(qubits, major_state[:dim]):
-                state_circuit.add_circbox(xbox, [flipped_qb])
+                state_circuit.add_circbox(xbox, [cast(UnitID, flipped_qb)])
         # Decompose boxes, add barriers to preserve circuit, add measures
-        state_circuit.add_barrier(all_qubits)
+        state_circuit.add_barrier(cast(List[UnitID], all_qubits))
+
         # add process circuit to measure
-        state_circuit.add_circbox(pbox, state_circuit.qubits)
+        state_circuit.add_circbox(pbox, cast(List[UnitID], state_circuit.qubits))
         DecomposeBoxes().apply(state_circuit)
-        state_circuit.add_barrier(all_qubits)
+        state_circuit.add_barrier(cast(List[UnitID], all_qubits))
         for q in measures:
             state_circuit.Measure(q, measures[q])
         # add to returned types
@@ -163,26 +169,25 @@ def get_full_transition_tomography_circuits(
 def calculate_correlation_matrices(
     results_list: List[BackendResult],
     states_info: List[StateInfo],
-    correlations: List[List[Qubit]],
+    correlations: List[List[Node]],
 ) -> FullCorrelatedNoiseCharacterisation:
     """Calculate the calibration matrices corresponding to some pure noise from the results of running calibration
     circuits.
 
     :param results_list: List of result via BackendResult. Must be in the same order as the
         corresponding circuits given by prepared_states.
-    :type results_list: List[BackendResult]
-    :param states_info: Each StateInfo object containts the state prepared via a binary
+    :param states_info: Each StateInfo object contains the state prepared via a binary
         representation and the qubit_to_bit_map for the corresponding state circuit.
-    :type states_info: List[StateInfo]
     :param correlations: List of dict corresponding to each prepared basis state
-    :type correlations: List[List[Qubit]]
 
     :return: Characterisation for pure noise given by process circuit
-    :rtype: FullCorrelatedNoiseCharacterisation
     """
 
+    def to_tuple(correlation_list: List[Node]) -> Tuple[Node, ...]:
+        return tuple(correlation_list)
+
     subsets_matrix_map = OrderedDict.fromkeys(
-        sorted(map(tuple, correlations), key=len, reverse=True)
+        sorted(map(to_tuple, correlations), key=len, reverse=True)
     )
     # ordered from largest to smallest via OrderedDict & sorted
     subset_dimensions = [len(subset) for subset in subsets_matrix_map]
@@ -221,27 +226,22 @@ def calculate_correlation_matrices(
     )
 
 
-#########################################
-### _compute_dot and helper functions ###
-###
-### With thanks to
-### https://math.stackexchange.com/a/3423910
-### and especially
-### https://gist.github.com/ahwillia/f65bc70cb30206d4eadec857b98c4065
-### on which this code is based.
+# _compute_dot and helper functions #
+#
+# With thanks to
+# https://math.stackexchange.com/a/3423910
+# and especially
+# https://gist.github.com/ahwillia/f65bc70cb30206d4eadec857b98c4065
+# on which this code is based.
 def _unfold(tens: np.ndarray, mode: int, dims: List[int]) -> np.ndarray:
     """
     Unfolds tensor into matrix.
 
     :param tens: Tensor with shape equivalent to dimensions
-    :type tens: np.ndarray
     :param mode: Specifies axis move to front of matrix in unfolding of tensor
-    :type mode: int
     :param dims: Gives shape of tensor passed
-    :type dims: List[int]
 
     :return: Matrix with shape (dims[mode], prod(dims[/mode]))
-    :rtype: np.ndarray
     """
     if mode == 0:
         return tens.reshape(dims[0], -1)
@@ -254,14 +254,10 @@ def _refold(vec: np.ndarray, mode: int, dims: List[int]) -> np.ndarray:
     Refolds vector into tensor.
 
     :param vec: Tensor with length equivalent to the product of dimensions given in dims
-    :type vec: np.ndarray
     :param mode: Axis tensor was unfolded along
-    :type mode: int
     :param dims: Shape of tensor
-    :type dims: List[int]
 
     :return: Tensor folded from vector with shape equivalent to dimensions given in dims
-    :rtype: np.ndarray
     """
     if mode == 0:
         return vec.reshape(dims)
@@ -277,13 +273,9 @@ def _compute_dot(submatrices: Iterable[np.ndarray], vector: np.ndarray) -> np.nd
     Multiplies the kronecker product of the given submatrices with given vector.
 
     :param submatrices: Submatrices multiplied
-    :type submatrices: Iterable[np.ndarray]
     :param vector: Vector multplied
-    :type vector: np.ndarray
 
     :return: Kronecker product of arguments
-    :rtype: np.ndarray
-
     """
     dims = [A.shape[0] for A in submatrices]
     vt = vector.reshape(dims)
@@ -302,17 +294,12 @@ def _bayesian_iteration(
     Transforms T corresponds to a Bayesian iteration, used to modfiy measurements.
 
     :param submatrices: submatrices to be inverted and applied to measurements.
-    :type submatrices: Iterable[np.ndarray]
     :param measurements: Probability distribution over some set of states to be amended.
-    :type measurements: np.ndarray
     :param t: Some transform to act on measurements.
-    :type t: np.ndarray
     :param epsilon: A stabilization parameter  to define an affine transformation for applicatoin
     to submatrices, eliminating zero probabilities.
-    :type epsilon: float
 
     :return: Transformed distribution vector.
-    :rtype: np.ndarray
     """
     # Transform t according to the Bayesian iteration
     # The parameter epsilon is a stabilization parameter which defines an affine
@@ -345,13 +332,9 @@ def _bayesian_iterative_correct(
     Converges when update states within tol range of previously tested states.
 
     :param submatrices: Matrices comprising the pure noise characterisation.
-    :type submatrices: Iterable[np.ndarray]
     :param input_vector: Vector corresponding to some counts distribution.
-    :type input_vector: np.ndarray
     :param tol: tolerance of closeness of found results
-    :type tol: float
     :param max_it: Maximum number of inversions attempted to correct results.
-    :type max_it: int
     """
     # based on method found in https://arxiv.org/abs/1910.00129
 
@@ -391,10 +374,7 @@ class CorrectionMethod(Enum):
         distribution from circuit and device.
 
         :param submatrices: Matrices comprising the pure noise characterisation.
-        :type submatrices: Iterable[np.ndarray]
         :param input_vector: Vector corresponding to some counts distribution.
-        :type input_vector: np.ndarray
-
         """
         try:
             subinverts = [np.linalg.inv(submatrix) for submatrix in submatrices]
@@ -420,13 +400,9 @@ class CorrectionMethod(Enum):
         Bayesian correction method.
 
         :param submatrices: Matrices comprising the pure noise characterisation.
-        :type submatrices: Iterable[np.ndarray]
         :param input_vector: Vector corresponding to some counts distribution.
-        :type input_vector: np.ndarray
         :param tol: tolerance of closeness of found results
-        :type tol: float
         :param max_it: Maximum number of inversions attempted to correct results.
-        :type max_it: int
         """
         return _bayesian_iterative_correct(
             submatrices, input_vector, tol=1e-5, max_it=500
@@ -442,13 +418,10 @@ def reduce_matrix(indices_to_remove: List[int], matrix: np.ndarray) -> np.ndarra
     because it is likely that unmeasured qubits will be in that state.
 
     :param indices_to_remove: Binary index of state matrix is mapping to be removed.
-    :type indices_to_remove: List[int]
     :param matrix: Transition matrix where indices correspond to some binary state, to have some
     dimension removed.
-    :type matrix: np.ndarray
 
     :return: Transition matrix with removed entries.
-    :rtype: np.ndarray
     """
 
     new_n_qubits = int(log2(matrix.shape[0])) - len(indices_to_remove)
@@ -480,12 +453,9 @@ def reduce_matrices(
     """
     Removes some dimensions from some matrices.
     :param entries_to_remove: Via indexing, says which dimensions to remove from which indices.
-    :type entries_to_remove: List[Tuple[int, int]]
     :param matrices: All matrices to have dimensions removed.
-    :type matrices: List[np.ndarray]
 
     :return: Matrices with some dimensions removed.
-    :rtype: List[np.ndarray]
     """
     organise: Dict[int, List[int]] = {k: [] for k in range(len(matrices))}
     for unused in entries_to_remove:
@@ -505,12 +475,9 @@ def get_single_matrix(
     """
     Returns a correction matrix just for the index given.
     :param entry_to_keep: Which matrix and indexing to return a correction matrix for.
-    :type entries_to_keep: Tuple[int, int]
     :param matrices: All matrices to find returned matrix from.
-    :type matrices: List[np.ndarray]
 
     :return: Matrix for correcting given entry.
-    :rtype: List[np.ndarray]
     """
     mat = matrices[entry_to_keep[0]]
     all_indices = list(range(int(log2(mat.shape[0]))))
@@ -529,11 +496,8 @@ def correct_transition_noise(
     matrices in noise_characterisation is applied to it
 
     :param result: BackendResult object to be negated by pure noise object.
-    :type result: BackendResult
     :param bit_qb_info: Used to permute corresponding BackendResult object so counts order matches noise characterisation.
-    :type bit_qb_info: Tuple[Dict[Qubit, Bit], Dict[Bit, Qubit]]
     :param noise_characterisation: Object holding all required information for some full noise characterisation of correlated subsets.
-    :type noise_characterisation: FullCorrelatedNoiseCharacterisation
     """
 
     final_measures_qb_map = bit_qb_info[0]
