@@ -1,12 +1,14 @@
-from abc import ABC
-import numpy as np
+from abc import ABC, abstractmethod
 from copy import deepcopy
+
+import numpy as np
 from numpy.typing import NDArray
 
 
 class SignalFilter(ABC):
     """Base class for signal filtering."""
 
+    @abstractmethod
     def filter(self, fft_result_val_grid: NDArray[np.float64]) -> NDArray[np.float64]:
         """Method transforming array of floats into filtered array of
         floats."""
@@ -22,7 +24,6 @@ class SmallCoefficientSignalFilter(SignalFilter):
         """Initialisation method.
 
         :param tol: Value below which coefficients should be reduced to 0
-        :type tol: float
         """
         self.tol = tol
 
@@ -31,9 +32,7 @@ class SmallCoefficientSignalFilter(SignalFilter):
         they are less than `tol`.
 
         :param fft_result_val_grid: Grid of values.
-        :type fft_result_val_grid: NDArray[float]
         :return: Grid of values with values less than `tol` reduced to 0.
-        :rtype: NDArray[float]
         """
 
         mitigated_fft_result_val_grid = deepcopy(fft_result_val_grid)
