@@ -12,23 +12,25 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .mittask import (
-    MitTask,
-    IOTask,
-    CircuitShots,
-    Wire,
-)
-from typing import List, Tuple, Union, OrderedDict, cast, Sequence
-from .task_graph import TaskGraph
-from pytket.backends import Backend, ResultHandle
-from pytket.backends.backendresult import BackendResult
-import networkx as nx  # type: ignore
 import inspect
 from copy import deepcopy
 from itertools import repeat
+from typing import List, OrderedDict, Sequence, Tuple, Union, cast
+
+import networkx as nx  # type: ignore
+import numpy as np
+from pytket import Bit, Circuit
+from pytket.backends import Backend, ResultHandle
+from pytket.backends.backendresult import BackendResult
 from pytket.utils.outcomearray import OutcomeArray
-from pytket import Bit
-import numpy as np  # type: ignore
+
+from .mittask import (
+    CircuitShots,
+    IOTask,
+    MitTask,
+    Wire,
+)
+from .task_graph import TaskGraph
 
 
 def backend_compile_circuit_shots_task_gen(
@@ -40,9 +42,7 @@ def backend_compile_circuit_shots_task_gen(
     Returns new wire wherein each circuit has been compiled.
 
     :param backend: Backend object from which compile circuit method is called
-    :type backend: Backend
     :param optimisation_level: Optimisation level of backend called.
-    :type optimmisation_level: int
     """
 
     def task(obj, circ_shots: List[CircuitShots]) -> Tuple[List[CircuitShots]]:
@@ -70,25 +70,21 @@ def backend_handle_task_gen(backend: Backend) -> MitTask:
     If different numbers of shots are passed, each circuit is run with the maximum number of shots.
 
     :param backend: Backend circuits are run through.
-    :type backend: Backend
     :return: Pure function that adds passes circuits to backend and gets handles.
-    :rtype: MitTask
     """
 
     def task(obj, circuit_wires: List[CircuitShots]) -> Tuple[List[ResultHandle]]:
         """
         :param circuit_wires: Circuits to be run on backend, number of shots to run of each.
-        :type circuit_wires: List[CircuitShots]
 
         :return: ResultHandles from process_circuits method.
-        :rtype: Tuple[List[ResultHandle]]
         """
 
         if len(circuit_wires) != 0:
             circs, shots = map(list, zip(*circuit_wires))
 
             results = backend.process_circuits(
-                circs, n_shots=cast(Sequence[int], shots)
+                cast(List[Circuit], circs), n_shots=cast(Sequence[int], shots)
             )
 
             return (results,)
@@ -106,21 +102,17 @@ def backend_res_task_gen(backend: Backend) -> MitTask:
     the backend the task is defined by.
 
     :param backend: backend holding results for handles.
-    :type backend: Backend
     """
 
     def task(obj, handles: List[ResultHandle]) -> Tuple[List[BackendResult]]:
+        """
+        :param handles: ResultHandle objects previously produced from backend.
 
+        :return: For each ResultHandle in handles, a BackendResult object retrieved from backend.
+        """
         results = backend.get_results(handles)
 
         return (results,)
-        """
-        :param handles: ResultHandle objects previously produced from backend.
-        :type handles: List[ResultHandle]
-
-        :return: For each ResultHandle in handles, a BackendResult object retrieved from backend.
-        :rtype: Tuple[List[BackendResult]]
-        """
 
     return MitTask(
         _label="HandlesToResults", _n_in_wires=1, _n_out_wires=1, _method=task
@@ -142,9 +134,7 @@ class MitRes(TaskGraph):
 
 
         :param backend: Pytket backend default constructor which tasks are generated from.
-        :type backend: Backend
         :param label: Name for identification of MitRes object.
-        :type label: str
         """
         # set member variables
         self._label = _label
@@ -174,10 +164,8 @@ class MitRes(TaskGraph):
         return type is Tuple[List[CircuitShots]].
 
         :param task: MitTask or TaskGraph object for checking wire numbers of.
-        :type task: Union[MitTask, Taskgraph]
 
         :return: True if task is suitable for prepending, False if not.
-        :rtype: bool
         """
         sig = inspect.signature(task.run)
         params = list(sig.parameters.values())
@@ -198,10 +186,8 @@ class MitRes(TaskGraph):
         return type is Tuple[List[BackendResult]].
 
         :param task: MitTask or TaskGraph object for checking wire numbers of.
-        :type task: Union[MitTask, Taskgraph]
 
         :return: True if task is suitable for apppending, False if not.
-        :rtype: bool
         """
         sig = inspect.signature(task.run)
         params = list(sig.parameters.values())
@@ -216,11 +202,18 @@ class MitRes(TaskGraph):
     def __str__(self) -> str:
         return f"<MitRes::{self._label}>"
 
-    def __call__(self, circuits_wire: List[List[CircuitShots]], characterisation: dict = {}) -> Tuple[List[BackendResult]]:  # type: ignore[override]
+    def __call__(  # type: ignore[override]
+        self,
+        circuits_wire: List[List[CircuitShots]],
+        cache: bool = False,
+        characterisation: dict = {},
+    ) -> Tuple[List[BackendResult]]:
         return cast(
             Tuple[List[BackendResult]],
             super().run(
-                cast(List[Wire], circuits_wire), characterisation=characterisation
+                cast(List[Wire], circuits_wire),
+                cache=cache,
+                characterisation=characterisation,
             ),
         )
 
@@ -229,9 +222,7 @@ class MitRes(TaskGraph):
         Returns a MitRes object from a TaskGraph object.
 
         :param task_graph: TaskGraph object to copy tasks from.
-        :type task_graph: TaskGraph
         :return: Copied TaskGraph as MitRes
-        :rtype: MitRes
         """
         if task_graph.n_in_wires != 1 or task_graph.n_out_wires != 1:
             raise TypeError(
@@ -274,7 +265,6 @@ class MitRes(TaskGraph):
         Not permitted for MitRes, raises TypeError.
 
         :param task: New task to be added in parallel.
-        :type task: MitTask
         """
         raise TypeError("MitRes.parallel forbidden.")
 
@@ -286,7 +276,6 @@ class MitRes(TaskGraph):
 
 
         :param num_wires: Number of edges to add between input and output vertices.
-        :type num_wires: int
         """
         raise TypeError("MitRes.add_n_wires forbidden.")
 
@@ -297,7 +286,12 @@ class MitRes(TaskGraph):
         """
         raise TypeError("MitRes.add_wire forbidden.")
 
-    def run(self, circuit_shots: List[CircuitShots], characterisation: dict = {}) -> List[BackendResult]:  # type: ignore[override]
+    def run(  # type: ignore[override]
+        self,
+        circuit_shots: List[CircuitShots],
+        cache: bool = False,
+        characterisation: dict = {},
+    ) -> List[BackendResult]:
         """
         Overloaded run method from TaskGraph class to add type checking.
         A single experiment is defined by a Tuple containing a circuit to be run
@@ -307,12 +301,10 @@ class MitRes(TaskGraph):
 
         :param circuit_shotss: Each tuple in circuit_wires contains a Circuit to run on
             internal backends and the number of shots to take of said circuit.
-        :type circuit_shots: List[CircuitShots]
 
         :return: A BackendResult object for each combination of circuit and shots.
-        :rtype: List[BackendResult]
         """
-        return self([circuit_shots], characterisation)[0]
+        return self([circuit_shots], cache, characterisation)[0]
 
 
 def split_shots_task_gen(max_shots: int) -> MitTask:
@@ -322,9 +314,7 @@ def split_shots_task_gen(max_shots: int) -> MitTask:
     shots matches that initially requested.
 
     :param max_shots: The maximum number of shots per job.
-    :type max_shots: int
     :return: A task dividing the job into several smaller ones.
-    :rtype: MitTask
     """
 
     def task(
@@ -336,10 +326,8 @@ def split_shots_task_gen(max_shots: int) -> MitTask:
 
         :param circuit_wires: A list of the circuits to be run, and the number
         of shots to be taken.
-        :type circuit_wires: List[CircuitShots]
         :return: A new list of circuits, some of which repeat with a smaller
         number of shots.
-        :rtype: Tuple[List[CircuitShots], List[int]]
         """
 
         # A new list of circuits and shots
@@ -385,7 +373,6 @@ def group_shots_task_gen() -> MitTask:
     by split_shots_task_gen.
 
     :return: Task merging jobs which correspond to the same circuit.
-    :rtype: MitTask
     """
 
     def task(
@@ -396,14 +383,11 @@ def group_shots_task_gen() -> MitTask:
 
         :param results_wires: A list of results, some of which correspond to
         the same circuit.
-        :type results_wires: List[BackendResult]
         :param split_index: A list of indexes identifying which results
         correspond to which circuit. Those results with index i correspond to
         the circuit i.
-        :type split_index: List[int]
         :return: A new list of results, where results which correspond to the
         same circuit have been grouped together.
-        :rtype: Tuple[List[BackendResult]]
         """
 
         # A new list of results where some results have been merged.
@@ -439,11 +423,8 @@ def gen_shot_split_MitRes(backend: Backend, max_shots: int) -> MitRes:
     jobs if necessary.
 
     :param backend: The backend on which to run the jobs.
-    :type backend: Backend
     :param max_shots: The maximum number of shots that each job should request.
-    :type max_shots: int
     :return: A MitRes object.
-    :rtype: MitRes
     """
 
     _experiment_mitres = MitRes(backend=backend)
