@@ -12,16 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import networkx as nx  # type: ignore
-from typing import List, Union, OrderedDict, Tuple, cast, TYPE_CHECKING
-from .mittask import (
-    MitTask,
-    IOTask,
-    Wire,
-)
-from .graphviz import _taskgraph_to_graphviz
 from copy import copy, deepcopy
 from tempfile import NamedTemporaryFile
+from typing import TYPE_CHECKING, List, OrderedDict, Tuple, Union, cast
+
+import networkx as nx  # type: ignore
+
+from .graphviz import _taskgraph_to_graphviz
+from .mittask import (
+    IOTask,
+    MitTask,
+    Wire,
+)
 
 if TYPE_CHECKING:
     import graphviz as gv  # type: ignore
@@ -36,7 +38,6 @@ class TaskGraph:
     types of ports edges are attached to must match.
 
     :param _label: Name for identification of TaskGraph object.
-    :type _label: str
     """
 
     def __init__(
@@ -61,10 +62,8 @@ class TaskGraph:
         Returns a new TaskGraph object from another TaskGraph object.
 
         :param task_graph: TaskGraph object to copy tasks from.
-        :type task_graph: TaskGraph
 
         :return: Copied TaskGraph
-        :rtype: TaskGraph
         """
         self._task_graph = deepcopy(task_graph._task_graph)
         self._label = task_graph._label
@@ -121,10 +120,8 @@ class TaskGraph:
         internal task_graph attribute matches the number of in wires to the graph.
 
         :param task: Wrapped pure function to prepend to graph
-        :type task: Union[MitTask, "TaskGraph"]
 
         :return: True if prepend permitted
-        :rtype: bool
         """
         return task.n_out_wires == self.n_in_wires
 
@@ -134,10 +131,8 @@ class TaskGraph:
         internal task_graph attribute matches the number of out wires to the graph.
 
         :param task: Wrapped pure function to append to graph
-        :type task: Union[MitTask, "TaskGraph"]
 
         :return: True if append permitted
-        :rtype: bool
         """
         return task.n_in_wires == self.n_out_wires
 
@@ -153,7 +148,6 @@ class TaskGraph:
         and output vertex, with no type restrictions.
 
         :param num_wires: Number of edges to add between input and output vertices.
-        :type num_wires: int
         """
         for _ in range(num_wires):
             in_port = len(self._task_graph.out_edges(self._i, data=True))
@@ -176,7 +170,6 @@ class TaskGraph:
         New edges also added from the Input vertex to the task (any type permitted), ports ordered in arguments order.
 
         :param task: New task to be prepended.
-        :type task: MitTask
         """
         assert self.check_prepend_wires(task)
         # It's possible a single generated MitTask object could be used in different TaskGraph objects
@@ -201,7 +194,6 @@ class TaskGraph:
         New edges added from task to Output vertex (any type permitted), ports ordered in arguments order.
 
         :param task: New task to be appended.
-        :type task: MitTask
         """
         assert self.check_append_wires(task)
         # It's possible a single generated MitTask object could be used in different TaskGraph objects
@@ -223,7 +215,7 @@ class TaskGraph:
         with the _task_graph structure held inside the node.
         """
         check_for_decompose = True
-        while check_for_decompose == True:
+        while check_for_decompose:
             # get all nodes and iterate through them
             check_for_decompose = False
             node_list = list(nx.topological_sort(self._task_graph))
@@ -316,7 +308,6 @@ class TaskGraph:
         Adds new MitTask/TaskGraph to TaskGraph object in parallel. All task in edges wired as out edges from Input vertex. All task out_Edges wired as in edges to Output Vertex.
 
         :param task: New task to be added in parallel.
-        :type task: MitTask
         """
         task = copy(task)
         base_n_input_outs = len(self._task_graph.out_edges(self._i))
@@ -356,15 +347,11 @@ class TaskGraph:
 
         :param input_wires: Each Wire holds information assigned as data to an output edge
             from the input vertex of the _task_graph.
-        :type input_wires: List[Wire]
         :param cache: If True each Tasks output data is stored in an OrderedDict with the
-            Task.label_ attribute as its key.
-        :type cache: bool
+            Task._label attribute as its key.
 
 
         :return: Data from input edges to output vertex, assigned as wires.
-        :rtype: Tuple[List[Wire]]
-
         """
         for edge, wire in zip(
             self._task_graph.out_edges(self._i, data=True), input_wires
@@ -378,7 +365,7 @@ class TaskGraph:
         self.characterisation.update(characterisation)
         # clear cache of held data if required
         # also check that all mittask label are unique else dict will fail
-        if cache == True:
+        if cache:
             unique_labels = set()
             for task in node_list:
                 if task not in (self._i, self._o):
@@ -404,7 +391,7 @@ class TaskGraph:
             # run held task
             outputs = task(inputs)
             self.characterisation.update(task.characterisation)
-            if cache == True:
+            if cache:
                 self._cache[task._label] = (task, outputs)
             # assign outputs ot out_edges of task
             out_edges = self._task_graph.out_edges(task, data=True, keys=True)
@@ -423,7 +410,6 @@ class TaskGraph:
         :returns: Dictionary holding all output data from all MitTask.
             This is only full after run is called with the cache argument set
             to True. Keys are stored in graph topological order.
-        :rtype: Dict[str, Tuple[MitTask, List[Wire]]]
         """
         return self._cache
 
@@ -432,7 +418,6 @@ class TaskGraph:
         Return a visual representation of the DAG as a graphviz object.
 
         :returns:   Representation of the DAG
-        :rtype:     graphviz.DiGraph
         """
         return _taskgraph_to_graphviz(self._task_graph, None, self._label)
 
