@@ -11,30 +11,35 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from copy import copy, deepcopy
+from enum import Enum
+from math import isclose
+from typing import Dict, List, Tuple, Union, cast
+
+import matplotlib.pyplot as plt
+import numpy as np
+from numpy.polynomial.polynomial import Polynomial
+from pytket import Circuit, OpType, Qubit
 from pytket.backends import Backend
+from pytket.circuit import Node
+from pytket.pauli import Pauli, QubitPauliString
+from pytket.predicates import CompilationUnit
+from pytket.unit_id import UnitID, UnitType
+from pytket.utils import QubitPauliOperator
+from pytket.utils.operators import CoeffTypeAccepted
+from scipy.optimize import curve_fit  # type: ignore
+from sympy import Expr  # type: ignore
+
 from qermit import (
+    AnsatzCircuit,
     MitEx,
     MitRes,
-    ObservableTracker,
-    AnsatzCircuit,
     MitTask,
     ObservableExperiment,
+    ObservableTracker,
     TaskGraph,
 )
-from copy import copy
-from pytket.pauli import QubitPauliString  # type: ignore
-from enum import Enum
-import numpy as np  # type: ignore
-from scipy.optimize import curve_fit  # type: ignore
-from typing import List, Tuple, cast, Dict
-from pytket import Circuit, OpType
-from pytket.predicates import CompilationUnit  # type: ignore
-from pytket.utils import QubitPauliOperator
-import matplotlib.pyplot as plt  # type: ignore
-from numpy.polynomial.polynomial import Polynomial  # type: ignore
-from pytket.circuit import Node  # type: ignore
-from math import isclose
-
+from qermit.noise_model import NoiseModel, PauliErrorTranspile
 
 box_types = {
     OpType.CircBox,
@@ -51,23 +56,20 @@ class Folding(Enum):
 
     :return: Circuit with gate count increased to scale noise. The unitary implemented by the
         circuit has not changed.
-    :rtype: Circuit
     """
 
     # TODO: circ does not appear as input in docs
     # TODO Generalise with 'partial folding' to allow for non integer noise scaling
-    def circuit(circ: Circuit, noise_scaling: int, **kwargs) -> Circuit:
+    @staticmethod
+    def circuit(circ: Circuit, noise_scaling: int, **kwargs) -> List[Circuit]:
         """Noise scaling by circuit folding. In this case the folded circuit is of
         the form :math:`CC^{-1}CC^{-1}...C` where :math:`C` is the original circuit. As such noise may be scaled by
         odd integers. The Unitary implemented is unchanged by this process.
 
         :param circ: Original circuit to be folded.
-        :type circ: Circuit
         :param noise_scaling: Factor by which to scale the noise. This must be an odd integer.
-        :type noise_scaling: int
         :raises ValueError: Raised if the amount by which the noise should be scaled is not an odd integer.
         :return: Folded circuit implementing identical unitary to the initial circuit.
-        :rtype: Circuit
         """
 
         # Raise if the amount by which the noise should be scaled is not an odd integer
@@ -79,7 +81,9 @@ class Folding(Enum):
         folded_circ = circ.copy()
         for _ in range(noise_scaling // 2):
             # Add barrier between circuit and its inverse
-            folded_circ.add_barrier(folded_circ.qubits + folded_circ.bits)
+            folded_circ.add_barrier(
+                cast(List[UnitID], folded_circ.qubits + folded_circ.bits)
+            )
 
             # Add inverse circuit by iterating though commands and inverting them
             for gate in reversed(circ.get_commands()):
@@ -91,7 +95,9 @@ class Folding(Enum):
                     folded_circ.add_gate(gate.op.dagger, gate.args)
 
             # Add barrier between circuit and its inverse
-            folded_circ.add_barrier(folded_circ.qubits + folded_circ.bits)
+            folded_circ.add_barrier(
+                cast(List[UnitID], folded_circ.qubits + folded_circ.bits)
+            )
 
             # Add original circuit
             for gate in circ.get_commands():
@@ -102,9 +108,10 @@ class Folding(Enum):
                 else:
                     folded_circ.add_gate(gate.op, gate.args)
 
-        return folded_circ
+        return [folded_circ]
 
-    def two_qubit_gate(circ: Circuit, noise_scaling: float, **kwargs) -> Circuit:
+    @staticmethod
+    def two_qubit_gate(circ: Circuit, noise_scaling: float, **kwargs) -> List[Circuit]:
         """Noise scaling by folding 2 qubit gates. It is implicitly
         assumed that the noise on the 2 qubit gates dominate. Two qubit gates
         :math:`G` are replaced by :math:`GG^{-1}G...G^{-1}G`. If
@@ -114,9 +121,7 @@ class Folding(Enum):
         as close as possible to but smaller then noise_scaling.
 
         :param circ: Original circuit to be folded.
-        :type circ: Circuit
         :param noise_scaling: Factor by which the noise should be scaled.
-        :type noise_scaling: int
 
         :raises ValueError: Raised if noise_scaling is less than 1.
         :raises ValueError: Raised if the noise cannot be scaled by
@@ -128,7 +133,6 @@ class Folding(Enum):
             approximate folding is allowed. Defaults to True.
 
         :return: Circuit with noise scaled.
-        :rtype: Circuit
         """
 
         if noise_scaling < 1:
@@ -144,8 +148,8 @@ class Folding(Enum):
             for i, cmd in enumerate(circ.get_commands())
             if (
                 not (cmd.op.type == OpType.Barrier)
-                and not (cmd.op.type in box_types)
-                and len(cmd.qubits) == 2
+                and (cmd.op.type not in box_types)
+                and (len(cmd.qubits) == 2)
             )
         }
 
@@ -207,28 +211,25 @@ class Folding(Enum):
             else:
                 folded_circuit.add_gate(gate.op, gate.args)
 
-        return folded_circuit
+        return [folded_circuit]
 
-    def gate(circ: Circuit, noise_scaling: float, **kwargs) -> Circuit:
+    @staticmethod
+    def gate(circ: Circuit, noise_scaling: float, **kwargs) -> List[Circuit]:
         """Noise scaling by gate folding. In this case gates :math:`G` are replaced at random
         with :math:`GG^{-1}G` until the number of gates is sufficiently scaled.
 
         :param circ: Original circuit to be folded.
-        :type circ: Circuit
         :param noise_scaling: Factor by which to increase the noise.
-        :type noise_scaling: float
 
         :key _allow_approx_fold: Allows for the noise to be increased by an amount close to that requested, as
             opposed to by exactly the amount requested.
             This is necessary as there are cases where the exact noise scaling cannot be achieved.
             This occurs due to the discrete
             amounts by which the noise can be increased (i.e. the discrete amount by which one gate increases the noise).
-        :type _allow_approx_fold: bool
 
         :raises ValueError: Raised if the requested noise scaling cannot be exactly achieved. This can be
             avoided by appropriately setting _allow_approx_fold.
         :return: Folded circuit implementing identical unitary to the initial circuit.
-        :rtype: Circuit
         """
 
         _allow_approx_fold = kwargs.get("_allow_approx_fold", 0)
@@ -317,19 +318,17 @@ class Folding(Enum):
         folded_c_dict["commands"] = folded_command_list
         folded_c = Circuit().from_dict(folded_c_dict)
 
-        return folded_c
+        return [folded_c]
 
-    def odd_gate(circ: Circuit, noise_scaling: int, **kwargs) -> Circuit:
+    @staticmethod
+    def odd_gate(circ: Circuit, noise_scaling: int, **kwargs) -> List[Circuit]:
         """Noise scaling by gate folding. In this case odd gates :math:`G` are
         replaced :math:`GG^{-1}G` until the number of gates is sufficiently
         scaled.
 
         :param circ: Original circuit to be folded.
-        :type circ: Circuit
         :param noise_scaling: Factor by which to increase the noise.
-        :type noise_scaling: float
         :return: Folded circuit implementing identical unitary to the initial circuit.
-        :rtype: Circuit
         """
 
         c_dict = circ.to_dict()
@@ -389,16 +388,45 @@ class Folding(Enum):
         c_dict.update({"commands": folded_command_list})
         folded_c = Circuit.from_dict(c_dict)
 
-        return folded_c
+        return [folded_c]
+
+    @staticmethod
+    def noise_aware(circ: Circuit, noise_scaling: int, **kwargs) -> List[Circuit]:
+        """Scale noise in a circuit by adding noisy gates as defined
+        by the given noise model.
+
+        :param circ: Circuit with noise to be scaled.
+        :param noise_scaling: Factor by which noise should be scaled.
+        :return: List of circuits with additional noise gates added.
+
+        :key noise_model: Noise model defining noise types and rates.
+            Defaults to noiseless model.
+        :key n_noisy_circuit_samples: The number of random noisy
+            circuits to generate. Defaults to 1.
+        """
+
+        noise_model: NoiseModel = kwargs.get("noise_model", NoiseModel(noise_model={}))
+        n_noisy_circuit_samples: int = kwargs.get("n_noisy_circuit_samples", 1)
+
+        scaled_noise_model: NoiseModel = noise_model.scale(
+            scaling_factor=noise_scaling - 1
+        )
+        error_transpiler = PauliErrorTranspile(noise_model=scaled_noise_model)
+
+        scaled_circ_list = []
+        for _ in range(n_noisy_circuit_samples):
+            scaled_circ = circ.copy()
+            error_transpiler.apply(scaled_circ)
+            scaled_circ_list.append(scaled_circ)
+
+        return scaled_circ_list
 
 
 def poly_exp_func(x: float, *params) -> float:
     """Definition of poly-exponential function for the purposes of fitting to data
 
     :param x: Value at which to evaluate function
-    :type x: float
     :return: Evaluation of poly-exponential function
-    :rtype: float
     """
     # Note that we use a list ending in 0 so that the constant in the polynomial is 0.
     # The constant can then be absorbed into the coefficient of the exponential.
@@ -409,9 +437,7 @@ def cube_root_func(x: float, *params) -> float:
     """Definition of cube root function for the purposes of fitting to data.
 
     :param x: Value at which to evaluate cube root function
-    :type x: float
     :return: Evaluation of cube root function
-    :rtype: float
     """
     y = x + params[2]
     return params[0] + params[1] * np.sign(y) * (np.abs(y)) ** (1 / 3)
@@ -421,23 +447,17 @@ class Fit(Enum):
     """Functions to fit to expectation values as they change with noise.
 
     :return: Extrapolation of expectation values to the zero noise limit.
-    :rtype: float
     """
 
     # TODO Consider adding adaptive exponential extrapolation
-    def cube_root(
-        self, x: List[float], y: List[float], _show_fit: bool, *args
-    ) -> float:
+    @staticmethod
+    def cube_root(x: List[float], y: List[float], _show_fit: bool, *args) -> float:
         """Fit data to a cube root function. This is to say a function of the form :math:`a + b(x+c)^{1/3}`.
 
         :param x: Noise scaling values.
-        :type x: List[float]
         :param y: Expectation values.
-        :type y: List[float]
         :param _show_fit: Plot data and resulting fitted function.
-        :type _show_fit: bool
         :return: Extrapolation of data to zero noise limit using the best fitting cube root function.
-        :rtype: float
         """
 
         # Fit data to cube root function
@@ -455,25 +475,21 @@ class Fit(Enum):
 
         return float(fit_to_zero)
 
+    @staticmethod
     def poly_exponential(
-        self, x: List[float], y: List[float], _show_fit: bool, deg: int
+        x: List[float], y: List[float], _show_fit: bool, deg: int
     ) -> float:
         """Fit data to a poly-exponential, which is to say a function of the
         form :math:`a+e^{z}`, where :math:`z` is a polynomial.
 
         :param x: Noise scaling values.
-        :type x: List[float]
         :param y: Expectation values.
-        :type y: List[float]
         :param _show_fit: Plot data and resulting fitted function.
-        :type _show_fit: bool
         :param deg: The degree of the polynomial in the exponential.
-        :type deg: int
         :raises ValueError: Raised if the degree of the polynomial
             inputted is negative, or too high to fit to the data.
         :return: Extrapolation of data to the zero noise limit using the best fitting
             poly-exponential function of the specified degree.
-        :rtype: float
         """
 
         # check that the degree of the polynomial is positive, and small enough to fit
@@ -517,44 +533,33 @@ class Fit(Enum):
 
         return float(fit_to_zero)
 
-    def exponential(
-        self, x: List[float], y: List[float], _show_fit: bool, *args
-    ) -> float:
+    @staticmethod
+    def exponential(x: List[float], y: List[float], _show_fit: bool, *args) -> float:
         """Fit data to an exponential function. This is to say a function of the form :math:`a+e^{(b+x)}`.
         Note that this is a special case of the poly-exponential function.
 
         :param x: Noise scaling values.
-        :type x: List[float]
         :param y: Expectation values.
-        :type y: List[float]
         :param _show_fit: Plot data and resulting fitting function.
-        :type _show_fit: bool
         :return: Extrapolation to zero noise limit using the best fitting exponential function.
-        :rtype: float
         """
 
         # As the exponential function is a special case of the
         # poly-exponential function, it is called here
-        return Fit.poly_exponential(self, x, y, _show_fit, 1)
+        return Fit.poly_exponential(x, y, _show_fit, 1)
 
-    def polynomial(
-        self, x: List[float], y: List[float], _show_fit: bool, deg: int
-    ) -> float:
+    @staticmethod
+    def polynomial(x: List[float], y: List[float], _show_fit: bool, deg: int) -> float:
         """Fit data to a polynomial function.
 
         :param x: Noise scaling values.
-        :type x: List[float]
         :param y: Expectation values.
-        :type y: List[float]
         :param _show_fit: Plot data and resulting fitting function.
-        :type _show_fit: bool
         :param deg: The degree of the function to fit to.
-        :type deg: int
         :raises ValueError: Raised if the degree of the polynomial is negative,
             or too high to fit the data to.
         :return: Extrapolation to zero noise limit using the best fitting polynomial
             function of the specified degree.
-        :rtype: float
         """
 
         # Raised if the degree of the polynomial requested is negative, or too high to fit the data to
@@ -575,45 +580,37 @@ class Fit(Enum):
             fit_x = linspace[0]
             fit_y = linspace[1]
 
-            plot_fit(x, y, fit_x, fit_y, fit_to_zero)
+            plot_fit(x, y, list(fit_x), list(fit_y), fit_to_zero)
 
         return float(fit_to_zero)
 
-    def linear(self, x: List[float], y: List[float], _show_fit: bool, *args) -> float:
+    @staticmethod
+    def linear(x: List[float], y: List[float], _show_fit: bool, *args) -> float:
         """Fit data to a linear function. This is to say a function of the form :math:`ax+b`.
         Note that this is a special case of the polynomial fitting function.
 
         :param x: Noise scaling values.
-        :type x: List[float]
         :param y: Expectation values.
-        :type y: List[float]
         :param _show_fit: Plot data and resulting fitted function.
-        :type _show_fit: bool
         :return: Extrapolation to zero noise limit using the best fitting linear function.
-        :rtype: float
         """
         # As this is a special case of a fit to a polynomial, the polynomial
         # fitting function is called here with a degree 1
-        return Fit.polynomial(self, x, y, _show_fit, 1)
+        return Fit.polynomial(x, y, _show_fit, 1)
 
-    def richardson(
-        self, x: List[float], y: List[float], _show_fit: bool, *args
-    ) -> float:
+    @staticmethod
+    def richardson(x: List[float], y: List[float], _show_fit: bool, *args) -> float:
         """Use richardson extrapolation. This amounts to fitting to a polynomial of
         degree one less than the number of data points.
 
         :param x: Noise scaling values.
-        :type x: List[float]
         :param y: Expectation values.
-        :type y: List[float]
         :param _show_fit: Plot data and resulting fitted function.
-        :type _show_fit: bool
         :return: Extrapolation to zero noise limit using Richardson extrapolation.
-        :rtype: float
         """
         # As this is a special case of the polynomial fitting function, the polynomial fitting
         # function is called here with degree one less than the number of data points.
-        return Fit.polynomial(self, x, y, _show_fit, len(x) - 1)
+        return Fit.polynomial(x, y, _show_fit, len(x) - 1)
 
 
 def plot_fit(
@@ -626,15 +623,10 @@ def plot_fit(
     """Plot expectation values at each noise level, and the fit to the data derived.
 
     :param x: Amounts by which the noise has been scaled
-    :type x: List[float]
     :param y: Expectation values at each noise level
-    :type y: List[float]
     :param fit_x: x coordinates at which to plot value of fitted function
-    :type fit_x: List[float]
     :param fit_y: Value of fitted function at each noise scaling
-    :type fit_y: List[float]
     :param fit_to_zero: The extrapolation of the fitted function to the zero noise limit
-    :type fit_to_zero: float
     """
 
     fig = plt.figure()
@@ -654,6 +646,7 @@ def digital_folding_task_gen(
     noise_scaling: float,
     _folding_type: Folding,
     _allow_approx_fold: bool,
+    **kwargs,
 ) -> MitTask:
     """
     Generates task transforming a circuit in order to amplify the noise. The noise
@@ -661,62 +654,135 @@ def digital_folding_task_gen(
 
     :param backend: This will be used to compile the circuit after folding to ensure
         that the gate set matches those available on the backend.
-    :type backend: Backend
     :param noise_scaling: The factor by which the noise is increased.
-    :type noise_scaling: float
     :param _folding_type: The means by which the noise should be increased.
-    :type _folding_type: Folding
     :param _allow_approx_fold:  Allows for the noise to be increased by an amount close to that requested, as
         opposed to by exactly the amount requested.
         This is necessary as there are cases where the exact noise scaling cannot be achieved.
         This occurs due to the discrete
         amounts by which the noise can be increased (i.e. the discrete amount by which one gate increases the noise).
-    :type _allow_approx_fold: bool
-
     """
 
     def task(
         obj,
         mitex_wire: List[ObservableExperiment],
-    ) -> Tuple[List[ObservableExperiment]]:
+    ) -> Tuple[List[ObservableExperiment], List[int]]:
         """Increase the noise levels impacting the circuit by increasing the
         number of gates. This preserves the action of the circuit.
 
         :param mitex_wire: List of experiments
-        :type mitex_wire: List[ObservableExperiment]
         :return: List of equivalent circuits, but with noise levels increased.
-        :rtype: Tuple[List[ObservableExperiment]]
+            Each noise scaling value may generate multiple circuits.
+            As such the return includes a lit of integers indicating to which
+            of the original circuits the new circuits belong.
         """
 
         folded_circuits = []
+        experiment_index = []
 
         # For each circuit in the input wire, extract the circuit, apply the fold,
         # and perform the necessary compilation.
-        for experiment in mitex_wire:
+        for index, experiment in enumerate(mitex_wire):
             # Apply the necessary folding method
-            zne_circ = _folding_type(experiment.AnsatzCircuit.Circuit, noise_scaling, _allow_approx_fold=_allow_approx_fold)  # type: ignore
+            zne_circ_list = _folding_type(
+                experiment.AnsatzCircuit.Circuit,
+                noise_scaling,
+                _allow_approx_fold=_allow_approx_fold,
+                **kwargs,
+            )  # type: ignore
 
-            # TODO: This additional compilation pass may result in the circuit noise being
-            # increased too much, and should be removed or better accounted for.
+            for zne_circ in zne_circ_list:
+                # TODO: This additional compilation pass may result in the circuit noise being
+                # increased too much, and should be removed or better accounted for.
 
-            # This compilation pass was added to account for the case that
-            # the inverse of a gate is not in the gateset of the backend.
-            backend.rebase_pass().apply(zne_circ)
+                # This compilation pass was added to account for the case that
+                # the inverse of a gate is not in the gateset of the backend.
+                backend.rebase_pass().apply(zne_circ)
 
-            folded_circuits.append(
-                ObservableExperiment(
-                    AnsatzCircuit=AnsatzCircuit(
-                        Circuit=zne_circ,
-                        Shots=experiment.AnsatzCircuit.Shots,
-                        SymbolsDict=experiment.AnsatzCircuit.SymbolsDict,
-                    ),
-                    ObservableTracker=experiment.ObservableTracker,
+                folded_circuits.append(
+                    ObservableExperiment(
+                        AnsatzCircuit=AnsatzCircuit(
+                            Circuit=zne_circ,
+                            Shots=experiment.AnsatzCircuit.Shots // len(zne_circ_list),
+                            SymbolsDict=deepcopy(experiment.AnsatzCircuit.SymbolsDict),
+                        ),
+                        ObservableTracker=deepcopy(experiment.ObservableTracker),
+                    )
                 )
+                experiment_index.append(index)
+
+        return (folded_circuits, experiment_index)
+
+    return MitTask(_label="DigitalFolding", _n_in_wires=1, _n_out_wires=2, _method=task)
+
+
+def merge_experiments_task_gen() -> MitTask:
+    """Generates task merging qubit pauli strings when they belong to the
+    same experiment.
+
+    :return: MitTask performing the merge.
+    """
+
+    def task(
+        obj, qpo_list: List[QubitPauliOperator], experiment_index_list: List[int]
+    ) -> Tuple[List[QubitPauliOperator]]:
+        """Merge a list of qubit pauli operators. Qubit pauli operators will
+        merged if they belong to the same experiment. The experiment each
+        qubit pauli operator belongs to is indicated by the index list.
+        Merging here means that the mean of the pauli string coefficients
+        is taken to be the coefficient of the qubit pauli string in the
+        new qubit pauli operator.
+
+        :param qpo_list: List of qubit pauli strings, some of which may belong
+            to the same experiment.
+        :param experiment_index_list: Indexes indicating to which experiment
+            the qubit pauli strings belong to.
+        :return: A list of merged qubit pauli operators.
+        """
+
+        # A dictionary mapping the experiment index to the dictionary
+        # of the merged qubit pauli operator.
+        index_to_merged_qpo_dict: Dict[
+            int, Dict[QubitPauliString, CoeffTypeAccepted]
+        ] = {}
+
+        # For each qubit pauli operator, sum the coefficients of the
+        # qubit pauli strings it contains.
+        for index, qpo in zip(experiment_index_list, qpo_list):
+            index_qpo_dict = index_to_merged_qpo_dict.get(
+                index, {qps: 0 for qps in qpo._dict.keys()}
             )
 
-        return (folded_circuits,)
+            if not index_qpo_dict.keys() == qpo._dict.keys():
+                raise Exception(
+                    "The qubit pauli strings being merged do not contain "
+                    + "matching qubit pauli strings. In particular "
+                    + f"{index_qpo_dict.keys()} and {qpo._dict.keys()} differ."
+                )
 
-    return MitTask(_label="DigitalFolding", _n_in_wires=1, _n_out_wires=1, _method=task)
+            # Sum qubit pauli string coefficients.
+            index_to_merged_qpo_dict[index] = {
+                qps: qpo._dict[qps] + index_qpo_dict[qps] for qps in qpo._dict.keys()
+            }
+
+        # For each index, find the mean coefficient.
+        for index, merged_qpo_dict in index_to_merged_qpo_dict.items():
+            index_to_merged_qpo_dict[index] = {
+                qps: coeff / experiment_index_list.count(index)
+                for qps, coeff in merged_qpo_dict.items()
+            }
+
+        # Convert the dictionary to a list and return.
+        return (
+            [
+                QubitPauliOperator(dictionary=index_to_merged_qpo_dict.get(index, {}))
+                for index in range(max(experiment_index_list) + 1)
+            ],
+        )
+
+    return MitTask(
+        _label="MergeExperiments", _n_in_wires=2, _n_out_wires=1, _method=task
+    )
 
 
 def extrapolation_task_gen(
@@ -725,13 +791,9 @@ def extrapolation_task_gen(
     """Generates task extrapolating to the zero noise limit using results from many folded circuits.
 
     :param noise_scaling_list: A list of the values by which the noise has been folded.
-    :type noise_scaling_list: List[float]
     :param _fit_type: The function used to fit to the resulting data.
-    :type _fit_type: Fit
     :param _show_fit: Plot data and resulting fitted function.
-    :type _show_fit: bool
     :param deg: The degree of polynomials used.
-    :type deg: int
     """
 
     def task(
@@ -740,13 +802,9 @@ def extrapolation_task_gen(
         """Returns expectation values corrected by extrapolation
 
         :param base_exp_list: List of expectation values corresponding to each experiment
-        :type base_exp_list: List[QubitPauliOperator]
         :return: Each element of this tuple is a list of expectations for each
         experiment, all with noise scaled by a fixed amount.
-        :rtype: Tuple[List[QubitPauliOperator]]
         """
-        # Combine noise folding levels with the unfolded experiment
-        all_fold_vals = [1, *noise_scaling_list]
 
         # Reformats to create list, where each list has fixed noise folding. Each element of
         # the list is a list of experiments
@@ -770,7 +828,7 @@ def extrapolation_task_gen(
                 QubitPauliOperator(
                     {
                         qpo_k: _fit_type(  # type: ignore
-                            obj, all_fold_vals, qpo_list_float[qpo_k], _show_fit, deg
+                            noise_scaling_list, qpo_list_float[qpo_k], _show_fit, deg
                         )
                         for qpo_k in qpo_list_float
                     }
@@ -781,7 +839,7 @@ def extrapolation_task_gen(
 
     return MitTask(
         _label="CollateZNEResults",
-        _n_in_wires=len(noise_scaling_list) + 1,
+        _n_in_wires=len(noise_scaling_list),
         _n_out_wires=1,
         _method=task,
     )
@@ -791,16 +849,14 @@ def copy_mitex_wire(wire: ObservableExperiment) -> ObservableExperiment:
     """Returns a single copy of the inputted wire
 
     :param wire: Pair of ansatz circuit and ObservableTracker
-    :type wire: ObservableExperiment
     :return: single copy of inputted wire
-    :rtype: ObservableExperiment
     """
 
     # Copy ansatz circuit
     new_ansatz_circuit = AnsatzCircuit(
-        Circuit=wire.AnsatzCircuit.Circuit.copy(),
-        Shots=copy(wire.AnsatzCircuit.Shots),
-        SymbolsDict=copy(wire.AnsatzCircuit.SymbolsDict),
+        Circuit=deepcopy(wire.AnsatzCircuit.Circuit),
+        Shots=deepcopy(wire.AnsatzCircuit.Shots),
+        SymbolsDict=deepcopy(wire.AnsatzCircuit.SymbolsDict),
     )
 
     # copy qps and instantiate new measurement setup
@@ -817,7 +873,6 @@ def gen_duplication_task(duplicates: int, **kwargs) -> MitTask:
     """Duplicate the inputted experiment wire
 
     :param duplicates: The number of times to duplicate the input wire.
-    :type duplicates: int
     """
 
     def task(
@@ -827,10 +882,8 @@ def gen_duplication_task(duplicates: int, **kwargs) -> MitTask:
         """Duplicate the inputted experiment wire
 
         :param mitex_wire: List of experiments
-        :type mitex_wire: List[ObservableExperiment]
         :raises ValueError: Raised if the number of duplications is less than 1
         :return: Many copies of the inputted wire
-        :rtype: Tuple[List[ObservableExperiment]]
         """
 
         # Raise error if the number of duplications requested is less than 1
@@ -860,25 +913,25 @@ def gen_duplication_task(duplicates: int, **kwargs) -> MitTask:
     )
 
 
-def qpo_node_relabel(qpo: QubitPauliOperator, node_map: Dict[Node, Node]):
+def qpo_node_relabel(
+    qpo: QubitPauliOperator,
+    node_map: Dict[Union[UnitID, Qubit, Node], Union[UnitID, Qubit, Node]],
+):
     """Relabel the nodes of qpo according to node_map
 
     :param qpo: Original qubit pauli operator
-    :type qpo: QubitPauliOperator
     :param node_map: Map between nodes
-    :type node_map: Dict[Node,Node]
     :return: Relabeled qubit pauli operator
-    :rtype: QubitPauliOperator
     """
 
     orig_qpo_dict = qpo._dict.copy()
-    new_qpo_dict = {}
+    new_qpo_dict: Dict[QubitPauliString, Union[int, float, complex, Expr]] = {}
     for orig_qps in orig_qpo_dict:
         orig_qps_dict = orig_qps.map
         new_qps_dict = {}
         for q in orig_qps_dict:
             new_qps_dict[node_map[q]] = orig_qps_dict[q]
-        new_qps = QubitPauliString(new_qps_dict)
+        new_qps = QubitPauliString(cast(Dict[Qubit, Pauli], new_qps_dict))
         new_qpo_dict[new_qps] = orig_qpo_dict[orig_qps]
 
     return QubitPauliOperator(new_qpo_dict)
@@ -892,26 +945,23 @@ def gen_initial_compilation_task(
     in the task graph.
 
     :param backend: Backend to compile to
-    :type backend: Backend
     :param optimisation_level: level of default compiler, defaults to 1
-    :type optimisation_level: int, optional
     """
 
     def task(
         obj, wire: List[ObservableExperiment]
-    ) -> Tuple[List[ObservableExperiment], Dict[Node, Node]]:
+    ) -> Tuple[List[ObservableExperiment], List[Dict[UnitID, UnitID]]]:
         """Performs initial compilation before folding. This is to ensure minimal compilation
         after folding, as this could disrupt by how much the noise is increased.
 
         :param wire: List of experiments
-        :type wire: List[ObservableExperiment]
         :return: List of experiments compiled to run on the inputted backend.
-        Additionally a dictionary describing how the nodes have been mapped
-        by compilation.
-        :rtype: Tuple[List[ObservableExperiment], Dict[Node, Node]]
+            Additionally a list of dictionaries describing how the nodes have
+            been mapped by compilation.
         """
 
         mapped_wire = []
+        node_map_list = []
 
         for obs_exp in wire:
             # Perform default compilation, tracking to which physical
@@ -923,6 +973,7 @@ def gen_initial_compilation_task(
                 optimisation_level=optimisation_level
             ).apply(cu)
             node_map = cu.final_map
+            node_map_list.append(node_map)
 
             # Alter the qubit pauli operator so that it maps to the new physical qubits.
             qpo = obs_exp[1]._qubit_pauli_operator
@@ -942,7 +993,7 @@ def gen_initial_compilation_task(
 
         return (
             mapped_wire,
-            node_map,
+            node_map_list,
         )
 
     return MitTask(
@@ -958,26 +1009,35 @@ def gen_qubit_relabel_task() -> MitTask:
     This should follow gen_initial_compilation_task
 
     :return: Task performing relabelling.
-    :rtype: MitTask
     """
 
     def task(
-        obj, qpo_list: List[QubitPauliOperator], compilation_map: Dict[Node, Node]
+        obj,
+        qpo_list: List[QubitPauliOperator],
+        compilation_map_list: List[Dict[Node, Node]],
     ) -> Tuple[List[QubitPauliOperator]]:
         """Use node map returned by compilation unit to undo the relabelling
         performed by gen_initial_compilation_task
 
         :param qpo_list: List of QubitPauliOperator
-        :type qpo_list: List[QubitPauliOperator]
-        :param compilation_map: Dictionary mapping nodes as returned by
-        gen_initial_compilation_task task
-        :type compilation_map: Dict[Node, Node]
+        :param compilation_map_list: List of Dictionaries mapping nodes as
+            returned by gen_initial_compilation_task task
         :return: List of QubitPauliOperator with relabeled nodes.
-        :rtype: Tuple[List[QubitPauliOperator]]
         """
 
-        node_map = {value: key for key, value in compilation_map.items()}
-        new_qpo_list = [qpo_node_relabel(qpo, node_map) for qpo in qpo_list]
+        new_qpo_list = []
+
+        for compilation_map, qpo in zip(compilation_map_list, qpo_list):
+            node_map = {value: key for key, value in compilation_map.items()}
+            new_qpo_list.append(
+                qpo_node_relabel(
+                    qpo,
+                    cast(
+                        Dict[Union[UnitID, Qubit, Node], Union[UnitID, Qubit, Node]],
+                        node_map,
+                    ),
+                )
+            )
 
         return (new_qpo_list,)
 
@@ -989,6 +1049,59 @@ def gen_qubit_relabel_task() -> MitTask:
     )
 
 
+def gen_noise_scaled_mitex(
+    backend: Backend,
+    noise_scaling: float,
+    **kwargs,
+) -> MitEx:
+    """Generates MitEx with noise scaled by the Qermit Folding methods.
+
+    :param backend: Backend on which circuits are run.
+    :param noise_scaling: Factor by which noise is scaled.
+
+    :return: MitEx with scaled noise.
+
+    :key experiment_mitres: MitRes on which circuits are run, defaults to
+        a MitRes wrapped around the given backend.
+    :key experiment_mitex: MitEx on which the circuits are run, defaults
+        to a MitEx wrapped around experiment_mitres
+    :key allow_approx_fold: Allow approximate folding which may occur as a
+        result of discreet folding due to adding gates.
+    :key folding_type: The noise scaling method to use.
+    """
+
+    _experiment_mitres = deepcopy(
+        kwargs.get(
+            "experiment_mitres",
+            MitRes(backend),
+        )
+    )
+
+    _experiment_mitex = deepcopy(
+        kwargs.get(
+            "experiment_mitex",
+            MitEx(backend, _label="ExperimentMitex", mitres=_experiment_mitres),
+        )
+    )
+
+    _allow_approx_fold = kwargs.get("allow_approx_fold", True)
+    _folding_type = kwargs.get("folding_type", Folding.circuit)
+
+    _fold_mitex = deepcopy(_experiment_mitex)
+    _fold_mitex._label = str(noise_scaling) + "FoldMitEx" + _fold_mitex._label
+
+    digital_folding_task = digital_folding_task_gen(
+        backend, noise_scaling, _folding_type, _allow_approx_fold, **kwargs
+    )
+
+    _fold_taskgraph = TaskGraph().from_TaskGraph(_fold_mitex)
+    _fold_taskgraph.add_wire()
+    _fold_taskgraph.prepend(digital_folding_task)
+    _fold_taskgraph.append(merge_experiments_task_gen())
+
+    return MitEx(backend).from_TaskGraph(_fold_taskgraph)
+
+
 # TODO: Backend does not appear as input in documentation
 def gen_ZNE_MitEx(backend: Backend, noise_scaling_list: List[float], **kwargs) -> MitEx:
     """Generates MitEx object which mitigates for noise using Zero Noise Extrapolation. This is the
@@ -996,69 +1109,42 @@ def gen_ZNE_MitEx(backend: Backend, noise_scaling_list: List[float], **kwargs) -
     extrapolating backwards. For further explanantion see https://arxiv.org/abs/2005.10921.
 
     :param backend: Backend on which the circuits are to be run.
-    :type backend: Backend
     :param noise_scaling_list: A list of the amounts by which the noise should be scaled.
-    :type noise_scaling_list: List[float]
     :return: MitEx object performing noise mitigation by ZNE.
-    :rtype: MitEx
     """
-    _experiment_mitres = copy(
-        kwargs.get(
-            "experiment_mitres",
-            MitRes(backend),
-        )
-    )
-
-    _experiment_mitex = copy(
-        kwargs.get(
-            "experiment_mitex",
-            MitEx(backend, _label="ExperimentMitex", mitres=_experiment_mitres),
-        )
-    )
-    _experiment_taskgraph = TaskGraph().from_TaskGraph(_experiment_mitex)
-
     _optimisation_level = kwargs.get("optimisation_level", 0)
     _show_fit = kwargs.get("show_fit", False)
-    _folding_type = kwargs.get("folding_type", Folding.circuit)
     _fit_type = kwargs.get("fit_type", Fit.linear)
     _deg = kwargs.get("deg", len(noise_scaling_list) - 1)
     _seed = kwargs.get("seed", None)
-    _allow_approx_fold = kwargs.get("allow_approx_fold", True)
 
     np.random.seed(seed=_seed)
 
-    for fold in noise_scaling_list:
-        _label = str(fold) + "FoldMitEx"
-        _fold_mitres = copy(
-            kwargs.get(
-                "experiment_mitres",
-                MitRes(backend),
-            )
+    _zne_taskgraph = TaskGraph().from_TaskGraph(
+        gen_noise_scaled_mitex(
+            noise_scaling=noise_scaling_list[0],
+            backend=backend,
+            **kwargs,
         )
-        _fold_mitex = copy(
-            kwargs.get(
-                "experiment_mitex",
-                MitEx(backend, _label=_label, mitres=_fold_mitres),
-            )
-        )
-        _fold_mitex._label = _label + _fold_mitex._label
-        digital_folding_task = digital_folding_task_gen(
-            backend, fold, _folding_type, _allow_approx_fold
-        )
-        _fold_mitex.prepend(digital_folding_task)
-        _experiment_taskgraph.parallel(_fold_mitex)
-
-    extrapolation_task = extrapolation_task_gen(
-        noise_scaling_list, _fit_type, _show_fit, _deg
     )
 
-    _experiment_taskgraph.prepend(gen_duplication_task(len(noise_scaling_list) + 1))
-    _experiment_taskgraph.append(extrapolation_task)
+    for fold in noise_scaling_list[1:]:
+        _zne_taskgraph.parallel(
+            gen_noise_scaled_mitex(
+                noise_scaling=fold,
+                backend=backend,
+                **kwargs,
+            )
+        )
 
-    _experiment_taskgraph.add_wire()
-
-    _experiment_taskgraph.prepend(
-        gen_initial_compilation_task(backend, _optimisation_level)
+    _zne_taskgraph.prepend(gen_duplication_task(len(noise_scaling_list)))
+    _zne_taskgraph.append(
+        extrapolation_task_gen(noise_scaling_list, _fit_type, _show_fit, _deg)
     )
-    _experiment_taskgraph.append(gen_qubit_relabel_task())
-    return MitEx(backend).from_TaskGraph(_experiment_taskgraph)
+
+    _zne_taskgraph.add_wire()
+
+    _zne_taskgraph.prepend(gen_initial_compilation_task(backend, _optimisation_level))
+    _zne_taskgraph.append(gen_qubit_relabel_task())
+
+    return MitEx(backend).from_TaskGraph(_zne_taskgraph)
